#ifndef __SSURFACE__
#define __SSURFACE__

#include "FresnelOutput_defs.h"

//#define DEBUG
#define FRESNEL_MAXSTEPS 100
#define USE_ROOT

#include <geomdefs.hh>
#include <assert.h>
#include <G4ThreeVector.hh>
class G4PhysicalVolume;

namespace G4FresnelLens{
class FSIntersectedSegment;
class SSurface __INHERITSTRING1 {
public:
    SSurface(double zpos=0., double rhomax=0.);
    virtual ~SSurface(){};

    //
    // Pure virtual methods
    virtual double PointOnSurf(double x)=0;
    virtual double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v)=0;
    virtual double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v,FSIntersectedSegment* is)=0;
    virtual double DistanceToSurfSafe(const G4ThreeVector& p)=0;
    virtual double FindNearestIntersection(const G4ThreeVector& p1, const G4ThreeVector& v,  FSIntersectedSegment* intSeg=0)=0;
    virtual double TestEdgeTolerance(int dir, const G4ThreeVector& p, const G4ThreeVector& v)=0;
    virtual EInside Inside(double rho, double z, FSIntersectedSegment* intSeg=0)=0;
    virtual bool Normal(const G4ThreeVector& p, bool up, G4ThreeVector* norm) { AbstractMethod(); return false; }
    virtual bool Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double up, G4ThreeVector* norm) { AbstractMethod(); return false; }
    virtual bool Point(int i, double step, double& rho, double& z)=0;

    //
    // redefinable virtual methods
    virtual double DistanceToSurfSafeUp(const G4ThreeVector& p) { return DistanceToSurfSafe(p); }
    virtual double DistanceToSurfSafeDown(const G4ThreeVector& p) { return DistanceToSurfSafe(p); }
    virtual double GetUpperZ(double rho)  { return PointOnSurf(rho); }
    virtual double GetBottomZ(double rho) { return PointOnSurf(rho); }
    virtual double GetEdgeZ()             { return PointOnSurf(fR); }
    virtual void print() { printf("abstract surface R=%g, z=%g\n", fR, fZpos); }

    virtual bool operator==(SSurface&) { AbstractMethod(); return false; }

    #ifdef USE_ROOT
    virtual void draw(const char* rootfile=0, const char* canvasfile=0, double shift=0., const char* prefix="", const char* option="RECREATE");
    #endif

    // getters
    double GetR()    { return fR; }
    double GetR1()   { return fR1; }
    double GetR2()   { return fR; }
    double GetZpos() { return fZpos; }
    double GetZ1()   { return fZ1; }
    double GetZ2()   { return fZ2; }
    virtual double GetMinZ() { return fZmin; }
    virtual double GetMaxZ() { return fZmax; }

    // plug for temporarily unimplemented methods
    void AbstractMethod() { fprintf(stderr, "Error, calling the unimplemented method. The method was not supposed to be used."); assert(false); }
protected:
    double fR1;
    double fR;
    double fZpos;
    double fZ1;
    double fZ2;
    double fZmin;
    double fZmax;

    static double kTolerance;
    static double kHalfTolerance;

#ifdef DEBUG
public:
    void setDebug(bool d=true) { theDebugSwitch=d; }
    bool getDebug() { return theDebugSwitch; }
protected:
    bool theDebugSwitch;
#endif
};

};
#endif
