#ifndef ShapeOfLens_HH
#define ShapeOfLens_HH

#include "FresnelOutput_defs.h"
#include "CylindricSurface.h"
#include "LensUtils.h"

#include <G4VSolid.hh>
#include <geomdefs.hh>

#define SLANTWALLS

class G4Tubs;
namespace G4FresnelLens{
class SSurface;
class ShapeOfLens: public G4VSolid  __INHERITSTRING
{
  public:  // with description

    ShapeOfLens(const G4String& pName,
                      G4double  pDz,
                      G4double  pR,
                      SSurface* upsurf, SSurface* downsurf);

    virtual ~ShapeOfLens();

     inline G4double GetCubicVolume();
     inline G4double GetSurfaceArea();
//     inline G4double CalculateSurfaceArea() const;
//
//     // Modifiers functions
//
//     inline void SetZHalfLength(G4double dz);
//     inline void SetRadiusMinusZ(G4double R1);
//     inline void SetRadiusPlusZ(G4double R2);

    // Solid standard methods

    //void ComputeDimensions(       G4VPVParamerisation p,
    //                        const G4Int               n,
    //                        const G4VPhysicalVolume*  pRep );
    G4bool CalculateExtent(const EAxis pAxis,
                           const G4VoxelLimits& pVoxelLimit,
                           const G4AffineTransform& pTransform,
                                 G4double& pmin, G4double& pmax) const;
    EInside Inside(const G4ThreeVector& p) const;
    G4ThreeVector SurfaceNormal( const G4ThreeVector& p) const;
    G4double DistanceToIn(const G4ThreeVector& p,
                          const G4ThreeVector& v) const;
    G4double DistanceToIn(const G4ThreeVector& p) const;
    G4double DistanceToOut(const G4ThreeVector& p,
                           const G4ThreeVector& v,
                           const G4bool calcNorm=G4bool(false),
                                 G4bool *validNorm=0,
                                 G4ThreeVector *n=0) const;
    G4double DistanceToOut(const G4ThreeVector& p) const;

    G4GeometryType GetEntityType() const;
    std::ostream& StreamInfo(std::ostream& os) const;

    G4ThreeVector GetPointOnSurface() const;
    // Visualisation functions

    void DescribeYourselfTo(G4VGraphicsScene& scene) const;
    G4Polyhedron* CreatePolyhedron() const;
    G4Polyhedron* GetPolyhedron () const;
    G4NURBS*      CreateNURBS() const;

    void GetCylinderLimits(double& r, double& z1, double& z2) { r=fR; z1=fZ1; z2=fZ2; }

public:  // without description
     ShapeOfLens(__void__&);
      // Fake default constructor for usage restricted to direct object
      // persistency for clients requiring preallocation of memory for
      // persistifiable objects.

protected:  // without description

//     mutable G4Polyhedron* fpPolyhedron;

private:
    EInside Inside(const G4ThreeVector& p, int* surface, FSIntersectedSegment* intseg=0) const;

    mutable G4double fSurfaceArea;
    G4double fCubicVolume;

    double fZ1, fZ2;
    G4double fDz, fR;
    G4Tubs* top_volume;

    double kTolerance;
    double kHalfTolerance;

    SSurface* fUpSurface;
    SSurface* fDownSurface;
    mutable CylindricSurface fCylinder;

    // save last position to calculate Normal after DistanceToIn faster
    mutable G4ThreeVector        theLastPositionOnSurface; // last position on surface, for normal calculation
    mutable FSIntersectedSegment theLastSurfaceSegment;    // for normal calculation
    mutable int                  theLastSurface;           // for normal calculation

    mutable G4ThreeVector        theLastPosition;          // last position, for "inside" definition
    mutable EInside              theLastInside;            // last inside definition
    mutable FSIntersectedSegment theLastInsideSegment;     // for normal calculation

    #ifdef SLANTWALLS
    private:
        void CorrectCylinderNormal(G4ThreeVector* normal) const;
        bool isSlant;
        double theSlantSigma;

    public:
        void SetSlant(bool slant=true) { isSlant=slant; }
        void SetSlantSigma(double sigma) { theSlantSigma=sigma; }
    #endif
};

//__________________________________________________________________________________
inline EInside ShapeOfLens::Inside(const G4ThreeVector& p) const {
    return Inside(p, 0);
}

//__________________________________________________________________________________
inline G4GeometryType ShapeOfLens::GetEntityType() const {
    return "FresnelLens";
}

//__________________________________________________________________________________
inline std::ostream& ShapeOfLens::StreamInfo(std::ostream& os) const {
    return os;
}

};
#endif
