#ifndef OptUserStackingAction_h
#define OptUserStackingAction_h 1
#include "OptSteppingAction.hh"

#include <G4UserStackingAction.hh>
#include <G4ThreeVector.hh>
#include <G4UserTrackingAction.hh>
#include <G4UserEventAction.hh>

#include "EsafMsgSource.hh"
#include "VirtualTelParm.hh"
class PhotonsOnPupil;
class Photon;
class G4ParticleDefinition;
class IdealFocalSurface;
class FocalPlane;
class EEvent;
class EVector;
class G4Track;
class G4Event;
class G4TrackInformation;
class ListPhotonsOnPupil;

class TGraph2D;

class OptUserStackingAction : public G4UserStackingAction, public G4UserTrackingAction, public G4UserEventAction, public EsafMsgSource, public VirtualTelParm  {
public:
    OptUserStackingAction(IdealFocalSurface* fs);
    ~OptUserStackingAction();

    virtual void PreUserTrackingAction(const G4Track*);
    virtual void PostUserTrackingAction(const G4Track*);
    virtual void BeginOfEventAction(const G4Event* anEvent);
    virtual void EndOfEventAction(const G4Event* anEvent);

    G4ClassificationOfNewTrack ClassifyNewTrack(const G4Track* aTrack);
    void NewStage();
    void PrepareNewEvent();
    void SetPupil(PhotonsOnPupil*);
 
    void SetNmax(int m) { fNmax=m; }
    void Progress(int, const char*);
    //Photon* GetCurrentPhoton()  { return fCurrentPhoton; }
    Photon* UpdateCurrentPhoton(G4Track*);

    double GetTimeFirst(){return fTimeFirstPhoton;}
    double GetTimeLast() {return fTimeLastPhoton;}
    void SetTrackHistory(int hist){fTrackHistory=hist;}
    OptSteppingAction* stepping;
    
    G4Track* NewPhoton(Photon*,int,int n=0);
    void     Reset();
private:

    bool GoToElectronics(Photon*);
    
    EVector  NewInteractionPoint(const Photon *p, double DZ) const;
    
    void CheckTrace(int id,const G4ThreeVector& pos,const G4ThreeVector& mom,double lambda,double time);

    G4ParticleDefinition* fPhotonDef;

    PhotonsOnPupil* fPupil;

    FocalPlane *fFocalPlane;
    IdealFocalSurface* fIdealFocalSurface;
    EEvent* fEv;

    float    fHuge;
    double   fTimeFirstPhoton;
    double   fTimeLastPhoton;
    
    int     fNReflElec;
    
    int     fNTracked;
    int     fNG4Tracked;
    int     fNmax;
    int     fNTotal;
    int     fNG4focalplane;
    int     fNiris;
    int     fNfocalplane;
    int     fNlost;
    int     fNoptics;
		int     fNwalls;
		int     fNearth;
    int     fProgress;
    int     fTrackHistory;
    int     forigin;
    bool    ftus;
    
    TGraph2D* f_before;
    TGraph2D* f_after;
    ListPhotonsOnPupil* fPhOnPupil;
};

#endif
