#include "G4TusGeometry.hh"
#include <G4Sphere.hh>
#include <G4Material.hh>
#include <G4MaterialTable.hh>
#include <G4Element.hh>
#include <G4ElementTable.hh>
#include <G4LogicalBorderSurface.hh>
#include <G4LogicalSkinSurface.hh>
#include <G4OpticalSurface.hh>
#include <G4Box.hh>
#include <G4LogicalVolume.hh>
#include <G4RotationMatrix.hh>
#include <G4ThreeVector.hh>
#include <G4Transform3D.hh>
#include <G4PVPlacement.hh>
#include <G4OpBoundaryProcess.hh>
#include <G4VisAttributes.hh>
#include <G4Cons.hh>
#include <G4SubtractionSolid.hh>
#include <G4IntersectionSolid.hh>
#include <G4UnionSolid.hh>
#include <G4Paraboloid.hh>
#include <G4Tubs.hh>
#include <G4Polyhedra.hh>

//#include <Geant4GM/volumes/Factory.h>
//#include <RootGM/volumes/Factory.h>


#include <TGeoManager.h>
#include <TMath.h>
#include <TEnv.h>

using namespace TMath;

G4TusGeometry::G4TusGeometry(){
    expHall_z = 5000.*mm;
    expHall_x = expHall_y = 1600.*mm;
    cout<<"OPTDETECTOR TUS"<<endl;
}

//_______________________________________________________________________________
G4TusGeometry::~G4TusGeometry(){

}

//_______________________________________________________________________________
G4VPhysicalVolume* G4TusGeometry::Construct(){
    G4double  a, z, density;
    G4int nelements;

    G4Element* Ag = new G4Element("Argentum", "Ag", z=47 , a=107.868*g/mole);

    G4Material* Vacuum =
    new G4Material("Galactic", z=1., a=1.01*g/mole,density= universe_mean_density, kStateGas,2.73*kelvin, 3.e-18*pascal);

    G4Material* metal= new G4Material("metal", density=10.49*g/cm3, nelements=1);
    metal->AddElement(Ag, 1);

    //metal
    const G4int nEntries = 2;
    G4double PhotonEnergy[nEntries] =
                {0.001*eV,50.*eV};/*0.2034*eV, 0.2068*eV, 0.2103*eV, 0.2139*eV,
                0.2177*eV, 0.2216*eV, 0.2256*eV, 0.2298*eV,
                0.2341*eV, 0.2386*eV, 0.2433*eV, 0.2481*eV,
                0.2532*eV, 0.2585*eV, 0.2640*eV, 0.2697*eV,
                0.2757*eV, 0.2820*eV, 0.2885*eV, 0.2954*eV,
                0.3026*eV, 0.3102*eV, 0.3181*eV, 0.3265*eV,
                0.3353*eV, 0.3446*eV, 0.3545*eV, 0.3649*eV,
                0.3760*eV, 0.3877*eV, 0.4002*eV, 0.9136*eV };*/

    G4double RefractiveIndex3[nEntries] =
            { 0., 0.};//, 0., 0.,0., 0., 0.,
            /*0., 0., 0., 0.,0., 0., 0.,
            0., 0., 0., 0.,0., 0., 0.,
            0., 0., 0., 0.,0., 0., 0.,
            0., 0., 0., 0. };*/

    G4double  Reflectivity3[nEntries] =
            { 0.99, 0.99};//, 0.99, 0.99,0.99, 0.99, 0.99,
            /*0.99, 0.99, 0.99, 0.99,0.99, 0.99, 0.99,
            0.99, 0.99, 0.99, 0.99,0.99, 0.99, 0.99,
            0.99, 0.99, 0.99, 0.99,0.99, 0.99, 0.99,
            0.99, 0.99, 0.99, 0.99 };*/

    G4MaterialPropertiesTable* myMPT3 = new G4MaterialPropertiesTable();
    myMPT3->AddProperty("RINDEX", PhotonEnergy, RefractiveIndex3, nEntries);
    myMPT3->AddProperty("REFLECTIVITY",PhotonEnergy,Reflectivity3,nEntries);
    metal->SetMaterialPropertiesTable(myMPT3);

    //Vacuum
    G4double RefractiveIndex1[nEntries] =
            { 1.00, 1.00 };/*, 1.00, 1.00, 1.00, 1.00, 1.00,
            1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00,
            1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00,
            1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00,
            1.00, 1.00, 1.00, 1.00 };*/

    G4double  Reflectivity1[nEntries] =
            { 0., 0.};/*, 0., 0.,0., 0., 0.,
            0., 0., 0., 0.,0., 0., 0.,
            0., 0., 0., 0.,0., 0., 0.,
            0., 0., 0., 0.,0., 0., 0.,
            0., 0., 0., 0. };*/

    G4MaterialPropertiesTable* myMPT1 = new G4MaterialPropertiesTable();
    myMPT1->AddProperty("RINDEX", PhotonEnergy, RefractiveIndex1, nEntries);
    myMPT1->AddProperty("REFLECTIVITY", PhotonEnergy,Reflectivity1 , nEntries);
    Vacuum->SetMaterialPropertiesTable(myMPT1);

    //world
    G4Box* expHall = new G4Box("World",expHall_x,expHall_y,expHall_z);
    G4LogicalVolume* expHall_log = new G4LogicalVolume(expHall,Vacuum,"World",0,0,0);
    G4VPhysicalVolume* expHall_phys = new G4PVPlacement(0,G4ThreeVector(0,0,0),expHall_log,"World",0,false,0);
    expHall_log->SetVisAttributes (G4VisAttributes::Invisible);

    // tus mirror
    const int ii = 11;
    double PRmin[ii] = {245.764,348.712,428.486,496.387,556.776,611.882,663.023,711.056,756.571,800.000,841.665};
    double PRmax[ii] = {347.563,427.083,494.773,554.977,609.918,660.908,708.802,754.188,797.496,839.047,879.090};


    //1-10 rings
    double dz1;
    double dz2;
    double dz;
    G4UnionSolid* segment1[ii-1];
    G4UnionSolid* segment2[ii];

    G4SubtractionSolid* segment11;
    for(int i=0; i<ii-1; i++){
        dz1 = ((PRmax[i]*PRmax[i])/(4.*(1500.+(10.*(i+1.)))))-(10.*(i+1.));
        dz2 = ((PRmin[i]*PRmin[i])/(4.*(1500.+(10.*(i+1.)))))-(10.*(i+1.));
        dz=(dz1-dz2);
        segment1[i] = OneSegment(PRmin[i],PRmax[i],dz*0.5,PRmin[i+1]);
    }

    //11 ring
    dz1 = ((PRmax[10]*PRmax[10])/(4.*(1500.+(10.*(10.+1.)))))-(10.*(10.+1.));
    dz2 = ((PRmin[10]*PRmin[10])/(4.*(1500.+(10.*(10.+1.)))))-(10.*(10.+1.));

    dz=(dz1-dz2);
    G4Paraboloid* par1 = new G4Paraboloid("par1",dz*0.5*mm,PRmin[10]*mm,PRmax[10]*mm);
    G4Tubs* tubs = new G4Tubs("tubs1",PRmin[10]*mm,PRmax[10]*mm,dz*0.5*mm,0.*deg,360.*deg);
    segment11 = new G4SubtractionSolid("kink",tubs,par1);


    segment2[0] = new G4UnionSolid("ring", segment1[0],segment1[1]);
    for(int j=1;j<9;j++)segment2[j] = new G4UnionSolid("ring", segment2[j-1],segment1[j+1]);
    segment2[9] = new G4UnionSolid("ring", segment2[8],segment11);


    dz=10.*mm;
    G4RotationMatrix*  pRot= new G4RotationMatrix();
    pRot->rotateX(180.*DegToRad());

    G4Paraboloid* seg0 = new G4Paraboloid("seg0",dz*0.5*mm,0.*mm,PRmin[0]*mm);
    segment2[10] = new G4UnionSolid("ring", segment2[9],seg0,pRot,G4ThreeVector(0,0,0));

    double p1[2]={-500.*mm,500.*mm};
    double p2[2]={0.*mm,0.*mm};
    double iside=sqrt(330.*330.-165.*165.)*mm;
    double oside=330.*mm;
    double p3[2]={iside,iside};

    G4Polyhedra* p = new G4Polyhedra("polly",0.,360.*deg,6,2,p1,p2,p3);

//     EInside ins1 = p->Inside(G4ThreeVector(290.*mm,0.,0.));
//     EInside ins2 = p->Inside(G4ThreeVector(0.*mm,290.*mm,0.));

    double sch = 0.1*sqrt(2.)*mm;
    G4IntersectionSolid* s[7];
     s[0] = new G4IntersectionSolid("s_0",segment2[10], p);
     s[1] = new G4IntersectionSolid("s_1",segment2[10], p,0,G4ThreeVector(0.,2.*iside+0.1,0.));
     s[2] = new G4IntersectionSolid("s_2",segment2[10], p,0,G4ThreeVector(0.,-2.*iside-0.1,0.));
     s[3] = new G4IntersectionSolid("s_3",segment2[10], p,0,G4ThreeVector(1.5*oside+sch,iside+sch,0.));
     s[4] = new G4IntersectionSolid("s_4",segment2[10], p,0,G4ThreeVector(1.5*oside+sch,-iside-sch,0.));
     s[5] = new G4IntersectionSolid("s_5",segment2[10], p,0,G4ThreeVector(-1.5*oside-sch,iside+sch,0.));
     s[6] = new G4IntersectionSolid("s_6",segment2[10], p,0,G4ThreeVector(-1.5*oside-sch,-iside-sch,0.));

    G4LogicalVolume* s_log[7];
    for(int i(0);i<7;i++)
    s_log[i] = new G4LogicalVolume(s[i],metal,Form("s%d_log",i),0,0,0);

    G4VPhysicalVolume* s_phys[7];

    for(int i(0);i<7;i++)
    s_phys[i] =new G4PVPlacement(0,G4ThreeVector(0.,0.,-1510.*mm),s_log[i],Form("mirror_phys%d",i),expHall_log,false,0);

    const G4int n=2;
    G4double PhotonEnergyS[n]    = { 0.01034*eV, 5.9136*eV};
    G4double RefractiveIndexS[n] = { 0.0, 0.0};
    G4double  ReflectivityS[n]   = { 0.99, 0.99};
    G4double efficiency[n]       = {0.99, 0.99};

    G4MaterialPropertiesTable* myST2 = new G4MaterialPropertiesTable();
    myST2->AddProperty("RINDEX",  PhotonEnergyS, RefractiveIndexS,n);
    myST2->AddProperty("REFLECTIVITY",PhotonEnergyS,ReflectivityS,n);
    myST2 -> AddProperty("EFFICIENCY",PhotonEnergyS,efficiency,n);


    //Optical Surface
    G4OpticalSurface* OpSurface[7];
    G4LogicalSkinSurface* Surface[7];
    for(int i(0);i<7;i++){
        OpSurface[i]= new G4OpticalSurface(Form("GlassSurface%d",i));
        Surface[i] = new G4LogicalSkinSurface(Form("GlassSurface%d",i),s_log[i],OpSurface[i]);
    //
        OpSurface[i]->SetType(dielectric_metal);
        OpSurface[i]->SetFinish(polished);
        OpSurface[i]->SetModel(glisur);

        OpSurface[i]->SetMaterialPropertiesTable(myST2);
    }


    G4LogicalVolume* absorber_log ;
    G4VPhysicalVolume* absorber_phys;
    double x= 120.*mm;
    G4Box* absorber = new G4Box("absorber",x,x,1.*mm);
    absorber_log = new G4LogicalVolume(absorber,Vacuum,"absorber",0,0,0);
    absorber_phys = new G4PVPlacement(0,G4ThreeVector(0.,0.,-14.*mm),absorber_log,"absorber",expHall_log,false,0);


//
//     G4VisAttributes* visatr= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
//     visatr->SetForceAuxEdgeVisible (1);
//     visatr->SetVisibility(true);
//     visatr->SetColour(0.99,0.8,0.8,.8);
//     FMirror_log->SetVisAttributes(visatr);
//     absorber_log->SetVisAttributes(visatr);

//    // Export VGM geometry to Root
//    //
//    Geant4GM::Factory g4Factory;
//    g4Factory.SetDebug(1);
//    g4Factory.Import(expHall_phys);
//
//    RootGM::Factory rtFactory;
//    rtFactory.SetDebug(1);
//    g4Factory.Export(&rtFactory);
//    gGeoManager->CloseGeometry();
//    gGeoManager->Export("SphericalLens.root");
    return expHall_phys;
}

//_______________________________________________________________________________
G4UnionSolid* G4TusGeometry::OneSegment(double  PRmin,double PRmax,double  PDz,double CRmax){
    G4Paraboloid* par1 = new G4Paraboloid("par1",PDz*mm,PRmin*mm,PRmax*mm);
    G4Tubs* tub1 = new G4Tubs("tubs1",PRmin*mm,PRmax*mm,PDz*mm,0.*deg,360.*deg);
    G4SubtractionSolid* sub_zub = new G4SubtractionSolid("kink",tub1,par1);

    G4Cons* cons = new G4Cons("cons", (PRmax-0.5)*mm,CRmax*mm,(PRmax-0.5)*mm,PRmax*mm, PDz*mm,0.*deg,360.*deg);
    G4Tubs* tub2 = new G4Tubs("tubs2",(PRmax-0.5)*mm,PRmax*mm,PDz*mm,0.*deg,360.*deg);
    G4SubtractionSolid* conus = new G4SubtractionSolid("conus",cons,tub2);
    G4UnionSolid* segment =  new G4UnionSolid("ring", sub_zub, conus);
    return segment;
}
