#include "OptPrimaryGeneratorAction.hh"



#include "Randomize.hh"
#include "TRandom3.h"
#include "DetectorGeometry.hh"
#include "Detector.hh"

#include "Pupil.hh"
#include "G4Event.hh"
#include "EVector.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4RunManager.hh"
#include "PhotonsOnPupil.hh"
#include "Photon.hh"
#include "TMath.h"
#include <iostream>

using namespace std;
using namespace TMath;
using namespace CLHEP;
//-----------------------------------------------------------------------------------------

OptPrimaryGeneratorAction::OptPrimaryGeneratorAction(bool active):
factive(active)
{
    SetSender("OptPrimaryGeneratorAction");
    G4int n_particle = 1;
    particleGun = new G4ParticleGun(n_particle);
    gRandom->SetSeed(99999);
    fThetaMin   = 0.;
    fThetaMax   = 0.;
    fPhiMin     = 0.;
    fPhiMax     = 360.;
    fDirection.set(0.,0.,1.);
    fEnergyMin  = 0.*MeV; //eV
    fEnergyMax  = 0.*MeV; //eV
    fEnergy     = 0.*MeV; //eV
    fTheta      = 0.*DegToRad();
    fPhi        = 0.*DegToRad();

    if (factive)Init();


}

//-----------------------------------------------------------------------------------------

OptPrimaryGeneratorAction::~OptPrimaryGeneratorAction()
{
    delete particleGun;

}
//-----------------------------------------------------------------------------------------
void OptPrimaryGeneratorAction::Init(){


    G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
    string p_type = Conf()->GetStr("PrimaryGeneratorAction.ParticleType");
    MsgForm(EsafMsg::Info,Form("Particle Type is %s",p_type.data()));
    G4ParticleDefinition* particle = particleTable->FindParticle(p_type.data());
    particleGun->SetParticleDefinition(particle);
    particleGun->SetParticleTime(0.0*ns);
    GenerateDirection();
    GenerateEnergy();

}

//-----------------------------------------------------------------------------------------
void OptPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
    if(!factive)return;
    string FillPosition = Conf()->GetStr("PrimaryGeneratorAction.GeneratePrimaries");
    if(!FillPosition.compare ("Circle"))
        FillCircle(anEvent);
    else if(!FillPosition.compare ("Point"))
        OnePoint(anEvent);
    else if(!FillPosition.compare ("Sphere"))
        FillSphere(anEvent);

}
//-----------------------------------------------------------------------------------------
void OptPrimaryGeneratorAction::GenerateDirection()
{
  //GenerateTheta
    fThetaMin = (double)Conf()->GetNum("PrimaryGeneratorAction.ThetaMin");
    fThetaMax = (double)Conf()->GetNum("PrimaryGeneratorAction.ThetaMax");

    if ( fThetaMin == fThetaMax )
         fTheta = fThetaMin*DegToRad();
    else fTheta = gRandom->Uniform(fThetaMin,fThetaMax)*DegToRad();
    fDirection.setTheta(fTheta);

//Generate Phi
    fPhiMin = (double)Conf()->GetNum("PrimaryGeneratorAction.PhiMin");
    fPhiMax = (double)Conf()->GetNum("PrimaryGeneratorAction.PhiMax");

    if ( fPhiMin == fPhiMax )
         fPhi = fPhiMin*DegToRad();
    else fPhi = gRandom->Uniform(fPhiMin,fPhiMax)*DegToRad();
    fDirection.setPhi(fPhi);
    fDirection.setMag(1.);
    particleGun->SetParticleMomentumDirection(fDirection);
}
//----------------------------------------------------------------------------------------
void OptPrimaryGeneratorAction::GenerateEnergy() {
         //
         // Genrate energy
         //
    fEnergyMin = (double)Conf()->GetNum("PrimaryGeneratorAction.EnergyMin");
    fEnergyMax = (double)Conf()->GetNum("PrimaryGeneratorAction.EnergyMax");
    if ( fEnergyMin == fEnergyMax )
        fEnergy = fEnergyMin;
    else fEnergy = gRandom->Uniform(fEnergyMin,fEnergyMax);
    particleGun->SetParticleEnergy(fEnergy*MeV);


}


void OptPrimaryGeneratorAction::OnePoint(G4Event* anEvent){
    double x =  (double)Conf()->GetNum("PrimaryGeneratorAction.Point.x");
    double y =  (double)Conf()->GetNum("PrimaryGeneratorAction.Point.y");
    particleGun->SetParticlePosition(G4ThreeVector(x*mm,y*mm,0.*mm));
    particleGun->GeneratePrimaryVertex(anEvent);

}

//-----------------------------------------------------------------------------------------
void OptPrimaryGeneratorAction::FillCircle(G4Event* anEvent){
    double x,y,phi,r;
    double rmin =  (double)Conf()->GetNum("PrimaryGeneratorAction.Circle.rmin");
    double rmax =  (double)Conf()->GetNum("PrimaryGeneratorAction.Circle.rmax");
    phi = gRandom->Uniform(0,TMath::TwoPi());
    r = gRandom->Uniform(rmin*mm,rmax*mm);
    x = r*Cos(phi);
    y = r*Sin(phi);
    particleGun->SetParticlePosition(G4ThreeVector(x,y,0.*mm));
    particleGun->GeneratePrimaryVertex(anEvent);
}


//-----------------------------------------------------------------------------------------
void OptPrimaryGeneratorAction::FillSphere(G4Event* anEvent){
    TVector3 p;
    double phi = gRandom->Uniform(0.,TwoPi());
    double th = gRandom->Uniform(Pi()-asin(1325./3143.57795946),Pi());
    p.SetMagThetaPhi(3143.57795946,th,phi);
    p.SetZ(p.Z()+3143.57795946+78.);
    particleGun->SetParticlePosition(G4ThreeVector(p.X(),p.Y(),p.Z()));
    particleGun->GeneratePrimaryVertex(anEvent);
}

